@@ -17,15 +17,10 @@
 """Manage change sets."""
 
 
-from __future__ import generators
-
-from cvs2svn_lib.boolean import *
-from cvs2svn_lib.set_support import *
 from cvs2svn_lib.common import InternalError
 from cvs2svn_lib.context import Ctx
 from cvs2svn_lib.symbol import Branch
 from cvs2svn_lib.symbol import Tag
-from cvs2svn_lib.cvs_item import CVSRevision
 from cvs2svn_lib.time_range import TimeRange
 from cvs2svn_lib.changeset_graph_node import ChangesetGraphNode
 
