@@ -17,11 +17,6 @@
 """The changeset dependency graph."""
 
 
-from __future__ import generators
-
-from cvs2svn_lib.boolean import *
-from cvs2svn_lib.set_support import *
-from cvs2svn_lib.context import Ctx
 from cvs2svn_lib.log import Log
 from cvs2svn_lib.changeset import RevisionChangeset
 from cvs2svn_lib.changeset import OrderedChangeset
@@ -56,14 +51,13 @@ class _NoPredNodes:
   All this repeated sorting is wasteful and unnecessary.  We should
   instead use a heap to output the changeset order, which would
   require O(lg N) work per add()/get() rather than O(1) and O(N lg N)
-  as in the current implementation [1].  But: (1) the heapq modules
-  was not added to Python until version 2.3; (2) the lame interface of
-  heapq doesn't allow an arbitrary compare function, so we would have
-  to store extra information in the array elements; (3) in practice,
-  the number of items in the list at any time is only a tiny fraction
-  of the total number of changesets; and (4) testing showed that the
-  heapq implementation is no faster than this one (perhaps because of
-  the increased memory usage).
+  as in the current implementation [1].  But: (1) the lame interface
+  of heapq doesn't allow an arbitrary compare function, so we would
+  have to store extra information in the array elements; (2) in
+  practice, the number of items in the list at any time is only a tiny
+  fraction of the total number of changesets; and (3) testing showed
+  that the heapq implementation is no faster than this one (perhaps
+  because of the increased memory usage).
 
   [1] According to Objects/listsort.txt in the Python source code, the
   Python list-sorting code is heavily optimized for arrays that have
@@ -79,14 +73,13 @@ class _NoPredNodes:
   def __len__(self):
     return len(self._nodes)
 
+  @staticmethod
   def _compare((node_1, changeset_1), (node_2, changeset_2)):
     """Define a (reverse) ordering on self._nodes."""
 
     return cmp(node_2.time_range, node_1.time_range) \
            or cmp(changeset_2, changeset_1)
 
-  _compare = staticmethod(_compare)
-
   def add(self, node):
     self._nodes.append( (node, self.changeset_db[node.id],) )
     self._sorted = False
