@@ -48,16 +48,10 @@ or branch.
 """
 
 
-from __future__ import generators
-
-import sys
 import os
 import stat
 import re
-import time
 
-from cvs2svn_lib.boolean import *
-from cvs2svn_lib.set_support import *
 from cvs2svn_lib import config
 from cvs2svn_lib.common import DB_OPEN_NEW
 from cvs2svn_lib.common import FatalError
@@ -65,7 +59,6 @@ from cvs2svn_lib.common import warning_prefix
 from cvs2svn_lib.common import error_prefix
 from cvs2svn_lib.common import IllegalSVNPathError
 from cvs2svn_lib.common import verify_svn_filename_legal
-from cvs2svn_lib.common import path_split
 from cvs2svn_lib.log import Log
 from cvs2svn_lib.context import Ctx
 from cvs2svn_lib.artifact_manager import artifact_manager
@@ -93,9 +86,8 @@ import cvs2svn_rcsparse
 # check that symbol definitions are reasonable).
 _valid_revision_re = re.compile(r'''
     ^
-    (?:[1-9]\d*\.)+     # Digit groups (but not starting with "0") with
-                        # trailing dots
-    (?:[1-9]\d*)        # And the last digit group
+    (?:\d+\.)+          # Digit groups with trailing dots
+    \d+                 # And the last digit group.
     $
     ''', re.VERBOSE)
 
@@ -336,13 +328,27 @@ class _SymbolDataCollector(object):
     # Canonicalize the revision number:
     revision = _branch_revision_re.sub(r'\1\2', revision)
 
+    old_name = name
     # Apply any user-defined symbol transforms to the symbol name:
     name = self.cvs_file.project.transform_symbol(
         self.cvs_file, name, revision
         )
 
-    # If the name transformed to None, then we ignore it:
-    if name is not None:
+    if name is None:
+      # Ignore symbol:
+      self.pdc.log_symbol_transform(old_name, None)
+      Log().verbose(
+          "   symbol '%s'=%s ignored in %s"
+          % (old_name, revision, self.cvs_file.filename,)
+          )
+    else:
+      if name != old_name:
+        self.pdc.log_symbol_transform(old_name, name)
+        Log().verbose(
+            "   symbol '%s'=%s transformed to '%s' in %s"
+            % (old_name, revision, name, self.cvs_file.filename,)
+            )
+
       # Verify that the revision number is valid:
       if _valid_revision_re.match(revision):
         # The revision number is valid; record it for later processing:
@@ -458,6 +464,7 @@ class _SymbolDataCollector(object):
 
     del self._symbol_defs
 
+  @staticmethod
   def rev_to_branch_number(revision):
     """Return the branch_number of the branch on which REVISION lies.
 
@@ -470,8 +477,6 @@ class _SymbolDataCollector(object):
       return None
     return revision[:revision.rindex(".")]
 
-  rev_to_branch_number = staticmethod(rev_to_branch_number)
-
   def rev_to_branch_data(self, revision):
     """Return the branch_data of the branch on which REVISION lies.
 
@@ -786,8 +791,7 @@ class _FileDataCollector(cvs2svn_rcsparse.Sink):
         self.cvs_file, self.pdc.trunk, cvs_items
         )
 
-    if Log().is_on(Log.DEBUG):
-      self._cvs_file_items.check_link_consistency()
+    self._cvs_file_items.check_link_consistency()
 
     # Tell the revision recorder about the file dependency tree.
     self.collect_data.revision_recorder.start_file(self._cvs_file_items)
@@ -904,8 +908,7 @@ class _FileDataCollector(cvs2svn_rcsparse.Sink):
     if self._file_imported:
       self._cvs_file_items.imported_remove_1_1(vendor_lod_items)
 
-    if Log().is_on(Log.DEBUG):
-      self._cvs_file_items.check_link_consistency()
+    self._cvs_file_items.check_link_consistency()
 
   def get_cvs_file_items(self):
     """Finish up and return a CVSFileItems instance for this file.
@@ -942,6 +945,10 @@ class _ProjectDataCollector:
     # and a tag in another.
     self.symbols = {}
 
+    # A map { (old_name, new_name) : count } indicating how many files
+    # were affected by each each symbol name transformation:
+    self.symbol_transform_counts = {}
+
   def get_symbol(self, name):
     """Return the Symbol object for the symbol named NAME in this project.
 
@@ -956,6 +963,29 @@ class _ProjectDataCollector:
       self.symbols[name] = symbol
     return symbol
 
+  def log_symbol_transform(self, old_name, new_name):
+    """Record that OLD_NAME was transformed to NEW_NAME in one file."""
+
+    try:
+      self.symbol_transform_counts[old_name, new_name] += 1
+    except KeyError:
+      self.symbol_transform_counts[old_name, new_name] = 1
+
+  def summarize_symbol_transforms(self):
+    if self.symbol_transform_counts and Log().is_on(Log.NORMAL):
+      log = Log()
+      log.normal('Summary of symbol transforms:')
+      transforms = self.symbol_transform_counts.items()
+      transforms.sort()
+      for ((old_name, new_name), count) in transforms:
+        if new_name is None:
+          log.normal('    "%s" ignored in %d files' % (old_name, count,))
+        else:
+          log.normal(
+              '    "%s" transformed to "%s" in %d files'
+              % (old_name, new_name, count,)
+              )
+
   def _process_cvs_file_items(self, cvs_file_items):
     """Process the CVSFileItems from one CVSFile."""
 
@@ -1307,6 +1337,8 @@ class CollectData:
           % (project.project_cvs_repos_path,)
           )
 
+    pdc.summarize_symbol_transforms()
+
     self.num_files += pdc.num_files
     Log().verbose('Processed', self.num_files, 'files')
 
