@@ -51,12 +51,7 @@ CVSItem
 """
 
 
-from __future__ import generators
-
-from cvs2svn_lib.boolean import *
-from cvs2svn_lib.set_support import *
 from cvs2svn_lib.context import Ctx
-from cvs2svn_lib.symbol import Trunk
 
 
 class CVSItem(object):
@@ -149,7 +144,7 @@ class CVSRevision(CVSItem):
 
   Members:
 
-    id -- (string) unique ID for this revision.
+    id -- (int) unique ID for this revision.
 
     cvs_file -- (CVSFile) CVSFile affected by this revision.
 
@@ -427,6 +422,23 @@ class CVSRevision(CVSItem):
       # prev_id is on the same LOD and this item closes that one:
       yield self.prev_id
 
+  def _get_branch_ids_recursively(self, cvs_file_items):
+    """Return the set of all CVSBranches that sprout from this CVSRevision.
+
+    After parent adjustment in FilterSymbolsPass, it is possible for
+    branches to sprout directly from a CVSRevision, or from those
+    branches, etc.  Return all branches that sprout from this
+    CVSRevision, directly or indirectly."""
+
+    retval = set()
+    branch_ids_to_process = list(self.branch_ids)
+    while branch_ids_to_process:
+      branch = cvs_file_items[branch_ids_to_process.pop()]
+      retval.add(branch)
+      branch_ids_to_process.extend(branch.branch_ids)
+
+    return retval
+
   def check_links(self, cvs_file_items):
     assert self.cvs_file == cvs_file_items.cvs_file
 
@@ -445,7 +457,10 @@ class CVSRevision(CVSItem):
       assert isinstance(first_on_branch, CVSBranch)
       assert first_on_branch.symbol == self.lod
       assert first_on_branch.next_id == self.id
-      assert first_on_branch.source_id == prev.id
+      cvs_revision_source = first_on_branch.get_cvs_revision_source(
+          cvs_file_items
+          )
+      assert cvs_revision_source.id == prev.id
       assert self.id in prev.branch_commit_ids
     else:
       # This revision follows another revision on the same LOD:
@@ -469,15 +484,20 @@ class CVSRevision(CVSItem):
       assert tag.source_id == self.id
       assert tag.source_lod == self.lod
 
-    branch_commit_ids = list(self.branch_commit_ids)
     for branch_id in self.branch_ids:
       branch = cvs_file_items[branch_id]
       assert isinstance(branch, CVSBranch)
       assert branch.source_id == self.id
       assert branch.source_lod == self.lod
+
+    branch_commit_ids = list(self.branch_commit_ids)
+
+    for branch in self._get_branch_ids_recursively(cvs_file_items):
+      assert isinstance(branch, CVSBranch)
       if branch.next_id is not None:
         assert branch.next_id in branch_commit_ids
         branch_commit_ids.remove(branch.next_id)
+
     assert not branch_commit_ids
 
     assert self.__class__ == cvs_revision_type_map[(
@@ -497,10 +517,6 @@ class CVSRevisionModification(CVSRevision):
 
   __slots__ = []
 
-  # Explicitly define pickle methods to get around a Python 2.2 bug:
-  __setstate__ = CVSRevision.__setstate__
-  __getstate__ = CVSRevision.__getstate__
-
   def get_cvs_symbol_ids_opened(self):
     return self.tag_ids + self.branch_ids
 
@@ -513,30 +529,18 @@ class CVSRevisionAdd(CVSRevisionModification):
 
   __slots__ = []
 
-  # Explicitly define pickle methods to get around a Python 2.2 bug:
-  __setstate__ = CVSRevisionModification.__setstate__
-  __getstate__ = CVSRevisionModification.__getstate__
-
 
 class CVSRevisionChange(CVSRevisionModification):
   """A CVSRevision that modifies a file that already existed on this LOD."""
 
   __slots__ = []
 
-  # Explicitly define pickle methods to get around a Python 2.2 bug:
-  __setstate__ = CVSRevisionModification.__setstate__
-  __getstate__ = CVSRevisionModification.__getstate__
-
 
 class CVSRevisionAbsent(CVSRevision):
   """A CVSRevision for which the file is nonexistent on this LOD."""
 
   __slots__ = []
 
-  # Explicitly define pickle methods to get around a Python 2.2 bug:
-  __setstate__ = CVSRevision.__setstate__
-  __getstate__ = CVSRevision.__getstate__
-
   def get_cvs_symbol_ids_opened(self):
     return []
 
@@ -546,10 +550,6 @@ class CVSRevisionDelete(CVSRevisionAbsent):
 
   __slots__ = []
 
-  # Explicitly define pickle methods to get around a Python 2.2 bug:
-  __setstate__ = CVSRevisionAbsent.__setstate__
-  __getstate__ = CVSRevisionAbsent.__getstate__
-
 
 class CVSRevisionNoop(CVSRevisionAbsent):
   """A CVSRevision that doesn't do anything.
@@ -562,10 +562,6 @@ class CVSRevisionNoop(CVSRevisionAbsent):
 
   __slots__ = []
 
-  # Explicitly define pickle methods to get around a Python 2.2 bug:
-  __setstate__ = CVSRevisionAbsent.__setstate__
-  __getstate__ = CVSRevisionAbsent.__getstate__
-
 
 # A map
 #
@@ -589,7 +585,7 @@ class CVSSymbol(CVSItem):
 
   Members:
 
-    id -- (string) unique ID for this item.
+    id -- (int) unique ID for this item.
 
     cvs_file -- (CVSFile) CVSFile affected by this item.
 
@@ -599,7 +595,9 @@ class CVSSymbol(CVSItem):
         this CVSSymbol.
 
     source_id -- (int) the ID of the CVSRevision or CVSBranch that is
-        the source for this item.
+        the source for this item.  This initially points to a
+        CVSRevision, but can be changed to a CVSBranch via parent
+        adjustment in FilterSymbolsPass.
 
     revision_recorder_token -- (arbitrary) a token that can be set by
         RevisionRecorder for the later use of RevisionReader.
@@ -624,6 +622,15 @@ class CVSSymbol(CVSItem):
     self.source_lod = source_lod
     self.source_id = source_id
 
+  def get_cvs_revision_source(self, cvs_file_items):
+    """Return the CVSRevision that is the ultimate source of this symbol."""
+
+    cvs_source = cvs_file_items[self.source_id]
+    while not isinstance(cvs_source, CVSRevision):
+      cvs_source = cvs_file_items[cvs_source.source_id]
+
+    return cvs_source
+
   def get_svn_path(self):
     return self.symbol.get_path(self.cvs_file.cvs_path)
 
@@ -637,7 +644,7 @@ class CVSBranch(CVSSymbol):
 
   Members:
 
-    id -- (string) unique ID for this item.
+    id -- (int) unique ID for this item.
 
     cvs_file -- (CVSFile) CVSFile affected by this item.
 
@@ -650,7 +657,9 @@ class CVSBranch(CVSSymbol):
         this CVSSymbol.
 
     source_id -- (int) id of the CVSRevision or CVSBranch from which
-        this branch sprouts.
+        this branch sprouts.  This initially points to a CVSRevision,
+        but can be changed to a CVSBranch via parent adjustment in
+        FilterSymbolsPass.
 
     next_id -- (int or None) id of first CVSRevision on this branch,
         if any; else, None.
@@ -774,10 +783,6 @@ class CVSBranchNoop(CVSBranch):
 
   __slots__ = []
 
-  # Explicitly define pickle methods to get around a Python 2.2 bug:
-  __setstate__ = CVSBranch.__setstate__
-  __getstate__ = CVSBranch.__getstate__
-
   def get_cvs_symbol_ids_opened(self):
     return []
 
@@ -800,7 +805,7 @@ class CVSTag(CVSSymbol):
 
   Members:
 
-    id -- (string) unique ID for this item.
+    id -- (int) unique ID for this item.
 
     cvs_file -- (CVSFile) CVSFile affected by this item.
 
@@ -810,7 +815,9 @@ class CVSTag(CVSSymbol):
         this CVSSymbol.
 
     source_id -- (int) the ID of the CVSRevision or CVSBranch that is
-        being tagged.
+        being tagged.  This initially points to a CVSRevision, but can
+        be changed to a CVSBranch via parent adjustment in
+        FilterSymbolsPass.
 
     revision_recorder_token -- (arbitrary) a token that can be set by
         RevisionRecorder for the later use of RevisionReader.
@@ -878,10 +885,6 @@ class CVSTagNoop(CVSTag):
 
   __slots__ = []
 
-  # Explicitly define pickle methods to get around a Python 2.2 bug:
-  __setstate__ = CVSTag.__setstate__
-  __getstate__ = CVSTag.__getstate__
-
 
 # A map
 #
