@@ -17,11 +17,6 @@
 """This module contains classes describing the sources of symbol fills."""
 
 
-from __future__ import generators
-
-from cvs2svn_lib.boolean import *
-from cvs2svn_lib.set_support import *
-from cvs2svn_lib.context import Ctx
 from cvs2svn_lib.common import InternalError
 from cvs2svn_lib.common import FatalError
 from cvs2svn_lib.common import SVN_INVALID_REVNUM
@@ -46,7 +41,7 @@ class FillSource:
   FillSource objects are able to compute the score for arbitrary
   source LODs and source revision numbers.
 
-  These objects are used by the symbol filler in SVNRepositoryMirror."""
+  These objects are used by the symbol filler in SVNOutputOption."""
 
   def __init__(self, cvs_path, symbol, node_tree):
     """Create a fill source.
@@ -146,13 +141,35 @@ class FillSource:
       return revision_ranges
 
   def get_subsources(self):
-    """Generate (cvs_path, FillSource) for all direct subsources."""
+    """Generate (CVSPath, FillSource) for all direct subsources."""
 
     if not isinstance(self._node_tree, SVNRevisionRange):
       for cvs_path, node in self._node_tree.items():
         fill_source = FillSource(cvs_path, self._symbol, node)
         yield (cvs_path, fill_source)
 
+  def get_subsource_map(self):
+    """Return the map {CVSPath : FillSource} of direct subsources."""
+
+    src_entries = {}
+
+    for (cvs_path, fill_subsource) in self.get_subsources():
+      src_entries[cvs_path] = fill_subsource
+
+    return src_entries
+
+  def __str__(self):
+    """For convenience only.  The format is subject to change at any time."""
+
+    return '%s(%s:%s)' % (
+        self.__class__.__name__, self._symbol, self.cvs_path,
+        )
+
+  def __repr__(self):
+    """For convenience only.  The format is subject to change at any time."""
+
+    return '%s%r' % (self, self._node_tree,)
+
 
 def get_source_set(symbol, range_map):
   """Return a FillSource describing the fill sources for RANGE_MAP.
@@ -164,9 +181,7 @@ def get_source_set(symbol, range_map):
   Use the SVNRevisionRanges from RANGE_MAP to create a FillSource
   instance describing the sources for filling SYMBOL."""
 
-  root_cvs_directory = Ctx()._cvs_file_db.get_file(
-      symbol.project.root_cvs_directory_id
-      )
+  root_cvs_directory = symbol.project.get_root_cvs_directory()
   fill_source = FillSource(root_cvs_directory, symbol, {})
 
   for cvs_symbol, svn_revision_range in range_map.items():
