@@ -17,9 +17,6 @@
 """This module contains the KeyGenerator class."""
 
 
-from cvs2svn_lib.boolean import *
-
-
 class KeyGenerator:
   """Generate a series of unique keys."""
 
@@ -40,11 +37,6 @@ class KeyGenerator:
 
     return self._last_id
 
-  def gen_key(self):
-    """Generate and return a previously-unused key, as a string."""
-
-    return '%x' % self.gen_id()
-
   def get_last_id(self):
     """Return the last id that was generated, as an integer."""
 
