@@ -17,15 +17,11 @@
 """This module defines the passes that make up a conversion."""
 
 
-from __future__ import generators
-
 import sys
 import os
 import shutil
 import cPickle
 
-from cvs2svn_lib.boolean import *
-from cvs2svn_lib.set_support import *
 from cvs2svn_lib import config
 from cvs2svn_lib.context import Ctx
 from cvs2svn_lib.common import warning_prefix
@@ -41,7 +37,6 @@ from cvs2svn_lib.pass_manager import Pass
 from cvs2svn_lib.serializer import PrimedPickleSerializer
 from cvs2svn_lib.artifact_manager import artifact_manager
 from cvs2svn_lib.cvs_file_database import CVSFileDatabase
-from cvs2svn_lib.metadata import Metadata
 from cvs2svn_lib.metadata_database import MetadataDatabase
 from cvs2svn_lib.project import read_projects
 from cvs2svn_lib.project import write_projects
@@ -75,36 +70,43 @@ from cvs2svn_lib.changeset_graph import ChangesetGraph
 from cvs2svn_lib.changeset_graph_link import ChangesetGraphLink
 from cvs2svn_lib.changeset_database import ChangesetDatabase
 from cvs2svn_lib.changeset_database import CVSItemToChangesetTable
-from cvs2svn_lib.svn_commit import SVNCommit
 from cvs2svn_lib.svn_commit import SVNRevisionCommit
 from cvs2svn_lib.openings_closings import SymbolingsLogger
 from cvs2svn_lib.svn_commit_creator import SVNCommitCreator
 from cvs2svn_lib.persistence_manager import PersistenceManager
 from cvs2svn_lib.collect_data import CollectData
-from cvs2svn_lib.process import run_command
+from cvs2svn_lib.process import call_command
 from cvs2svn_lib.check_dependencies_pass \
     import CheckItemStoreDependenciesPass
 from cvs2svn_lib.check_dependencies_pass \
     import CheckIndexedItemStoreDependenciesPass
 
 
-def sort_file(infilename, outfilename, options=''):
-  """Sort file INFILENAME, storing the results to OUTFILENAME."""
+def sort_file(infilename, outfilename, options=[]):
+  """Sort file INFILENAME, storing the results to OUTFILENAME.
+
+  OPTIONS is an optional list of strings that are passed as additional
+  options to the sort command."""
 
   # GNU sort will sort our dates differently (incorrectly!) if our
   # LC_ALL is anything but 'C', so if LC_ALL is set, temporarily set
   # it to 'C'
   lc_all_tmp = os.environ.get('LC_ALL', None)
   os.environ['LC_ALL'] = 'C'
-  command = '%s -T %s %s %s > %s' % (
-      Ctx().sort_executable, Ctx().tmpdir, options, infilename, outfilename
-      )
+
+  # The -T option to sort has a nice side effect.  The Win32 sort is
+  # case insensitive and cannot be used, and since it does not
+  # understand the -T option and dies if we try to use it, there is no
+  # risk that we use that sort by accident.
+  command = [
+      Ctx().sort_executable,
+      '-T', Ctx().tmpdir
+      ] + options + [
+      infilename
+      ]
+
   try:
-    # The -T option to sort has a nice side effect.  The Win32 sort is
-    # case insensitive and cannot be used, and since it does not
-    # understand the -T option and dies if we try to use it, there is
-    # no risk that we use that sort by accident.
-    run_command(command)
+    call_command(command, stdout=open(outfilename, 'w'))
   finally:
     if lc_all_tmp is None:
       del os.environ['LC_ALL']
@@ -112,11 +114,21 @@ def sort_file(infilename, outfilename, options=''):
       os.environ['LC_ALL'] = lc_all_tmp
 
   # On some versions of Windows, os.system() does not return an error
-  # if the command fails.  So add a little consistency test here that
+  # if the command fails.  So add little consistency tests here that
   # the output file was created and has the right size:
-  if not os.path.exists(outfilename) \
-     or os.path.getsize(outfilename) != os.path.getsize(infilename):
-    raise FatalError('Command failed: "%s"' % (command,))
+
+  if not os.path.exists(outfilename):
+    raise FatalError('Sort output file missing: %r' % (outfilename,))
+
+  if os.path.getsize(outfilename) != os.path.getsize(infilename):
+    raise FatalError(
+        'Sort input and output file sizes differ:\n'
+        '    %r (%d bytes)\n'
+        '    %r (%d bytes)' % (
+            infilename, os.path.getsize(infilename),
+            outfilename, os.path.getsize(outfilename),
+            )
+        )
 
 
 class CollectRevsPass(Pass):
@@ -518,9 +530,7 @@ class FilterSymbolsPass(Pass):
       cvs_file_items.refine_symbols()
       cvs_file_items.record_opened_symbols()
       cvs_file_items.record_closed_symbols()
-
-      if Log().is_on(Log.DEBUG):
-        cvs_file_items.check_link_consistency()
+      cvs_file_items.check_link_consistency()
 
       # Store whatever is left to the new file and update statistics:
       stats_keeper.record_cvs_file(cvs_file_items.cvs_file)
@@ -657,6 +667,7 @@ class InitializeChangesetsPass(Pass):
     if changeset_items:
       yield changeset_items
 
+  @staticmethod
   def compare_items(a, b):
       return (
           cmp(a.timestamp, b.timestamp)
@@ -665,8 +676,6 @@ class InitializeChangesetsPass(Pass):
                  [int(x) for x in b.rev.split('.')])
           or cmp(a.id, b.id))
 
-  compare_items = staticmethod(compare_items)
-
   def break_internal_dependencies(self, changeset_items):
     """Split up CHANGESET_ITEMS if necessary to break internal dependencies.
 
@@ -1668,7 +1677,8 @@ class SortSymbolsPass(Pass):
         artifact_manager.get_temp_file(config.SYMBOL_OPENINGS_CLOSINGS),
         artifact_manager.get_temp_file(
             config.SYMBOL_OPENINGS_CLOSINGS_SORTED),
-        options='-k 1,1 -k 2,2n -k 3')
+        options=['-k', '1,1', '-k', '2,2n', '-k', '3'],
+        )
     Log().quiet("Done")
 
 
