@@ -17,8 +17,6 @@
 """This module contains classes to set cvs2svn run options."""
 
 
-from __future__ import generators
-
 import sys
 import os
 import re
@@ -29,7 +27,6 @@ try:
 except AttributeError:
   my_getopt = getopt.getopt
 
-from cvs2svn_lib.boolean import *
 from cvs2svn_lib.version import VERSION
 from cvs2svn_lib import config
 from cvs2svn_lib.common import warning_prefix
@@ -56,11 +53,14 @@ from cvs2svn_lib.symbol_strategy import BranchIfCommitsRule
 from cvs2svn_lib.symbol_strategy import ExcludeRegexpStrategyRule
 from cvs2svn_lib.symbol_strategy import ForceBranchRegexpStrategyRule
 from cvs2svn_lib.symbol_strategy import ForceTagRegexpStrategyRule
+from cvs2svn_lib.symbol_strategy import ExcludeTrivialImportBranchRule
 from cvs2svn_lib.symbol_strategy import HeuristicStrategyRule
 from cvs2svn_lib.symbol_strategy import UnambiguousUsageRule
 from cvs2svn_lib.symbol_strategy import HeuristicPreferredParentRule
 from cvs2svn_lib.symbol_strategy import SymbolHintsFileRule
-from cvs2svn_lib.symbol_strategy import DefaultBasePathRule
+from cvs2svn_lib.symbol_strategy import TrunkPathRule
+from cvs2svn_lib.symbol_strategy import BranchesPathRule
+from cvs2svn_lib.symbol_strategy import TagsPathRule
 from cvs2svn_lib.symbol_transform import ReplaceSubstringsSymbolTransform
 from cvs2svn_lib.symbol_transform import RegexpSymbolTransform
 from cvs2svn_lib.symbol_transform import NormalizePathsSymbolTransform
@@ -97,6 +97,8 @@ history.
                              with --svnrepos)
       --bdb-txn-nosync       pass --bdb-txn-nosync to "svnadmin create" (for
                              use with --svnrepos)
+      --create-option=OPT    pass OPT to "svnadmin create" (for use with
+                             --svnrepos)
       --dumpfile=PATH        just produce a dumpfile; don't commit to a repos
       --dry-run              do not create a repository or a dumpfile;
                              just print what would happen.
@@ -122,13 +124,17 @@ history.
       --force-branch=REGEXP  force symbols matching REGEXP to be branches
       --force-tag=REGEXP     force symbols matching REGEXP to be tags
       --exclude=REGEXP       exclude branches and tags matching REGEXP
+      --keep-trivial-imports do not exclude branches that were only used for
+                             a single import (usually these are unneeded)
       --symbol-default=OPT   specify how ambiguous symbols are converted.
                              OPT is "heuristic" (default), "strict", "branch",
                              or "tag"
-      --no-cross-branch-commits   Prevent the creation of cross-branch commits
+      --keep-cvsignore       keep .cvsignore files (in addition to creating
+                             the analogous svn:ignore properties)
+      --no-cross-branch-commits   prevent the creation of cross-branch commits
       --retain-conflicting-attic-files   if a file appears both in and out of
                              the CVS Attic, then leave the attic version in a
-                             SVN directory called "Attic".
+                             SVN directory called "Attic"
       --username=NAME        username for cvs2svn-synthesized commits
       --cvs-revnums          record CVS revision numbers as file properties
       --mime-types=FILE      specify an apache-style mime.types file for
@@ -205,7 +211,8 @@ class RunOptions:
       self.opts, self.args = my_getopt(cmd_args, 'hvqs:p:', [
           "options=",
 
-          "svnrepos=", "existing-svnrepos", "fs-type=", "bdb-txn-nosync",
+          "svnrepos=", "existing-svnrepos",
+          "fs-type=", "bdb-txn-nosync", "create-option=",
           "dumpfile=",
           "dry-run",
 
@@ -215,7 +222,9 @@ class RunOptions:
           "encoding=", "fallback-encoding=",
           "symbol-transform=",
           "symbol-hints=",
-          "force-branch=", "force-tag=", "exclude=", "symbol-default=",
+          "force-branch=", "force-tag=", "exclude=",
+          "keep-trivial-imports", "symbol-default=",
+          "keep-cvsignore",
           "no-cross-branch-commits",
           "retain-conflicting-attic-files",
           "username=",
@@ -288,6 +297,7 @@ class RunOptions:
         self,
         project_cvs_repos_path,
         trunk_path=None, branches_path=None, tags_path=None,
+        initial_directories=[],
         symbol_transforms=None,
         symbol_strategy_rules=[],
         ):
@@ -298,12 +308,28 @@ class RunOptions:
     SymbolStrategyRules that will be applied to symbols in this
     project."""
 
+    initial_directories = [
+        path
+        for path in [trunk_path, branches_path, tags_path]
+        if path
+        ] + list(initial_directories)
+
+    symbol_strategy_rules = list(symbol_strategy_rules)
+
+    # Add rules to set the SVN paths for LODs depending on whether
+    # they are the trunk, tags, or branches:
+    if trunk_path is not None:
+      symbol_strategy_rules.append(TrunkPathRule(trunk_path))
+    if branches_path is not None:
+      symbol_strategy_rules.append(BranchesPathRule(branches_path))
+    if tags_path is not None:
+      symbol_strategy_rules.append(TagsPathRule(tags_path))
+
     id = len(self.projects)
     project = Project(
         id,
         project_cvs_repos_path,
-        trunk_path=trunk_path,
-        branches_path=branches_path, tags_path=tags_path,
+        initial_directories=initial_directories,
         symbol_transforms=symbol_transforms,
         )
 
@@ -370,11 +396,13 @@ class RunOptions:
     existing_svnrepos = False
     fs_type = None
     bdb_txn_nosync = False
+    create_options = []
     dump_only = False
     dumpfile = None
     use_rcs = False
     use_cvs = False
     use_internal_co = False
+    keep_trivial_imports = False
     symbol_strategy_default = 'heuristic'
     mime_types_file = None
     auto_props_file = None
@@ -431,11 +459,15 @@ class RunOptions:
         force_tag = True
       elif opt == '--exclude':
         symbol_strategy_rules.append(ExcludeRegexpStrategyRule(value))
+      elif opt == '--keep-trivial-imports':
+        keep_trivial_imports = True
       elif opt == '--symbol-default':
         if value not in ['branch', 'tag', 'heuristic', 'strict']:
           raise FatalError(
               '%r is not a valid option for --symbol_default.' % (value,))
         symbol_strategy_default = value
+      elif opt == '--keep-cvsignore':
+        ctx.keep_cvsignore = True
       elif opt == '--no-cross-branch-commits':
         ctx.cross_branch_commits = False
       elif opt == '--retain-conflicting-attic-files':
@@ -453,6 +485,8 @@ class RunOptions:
         fs_type = value
       elif opt == '--bdb-txn-nosync':
         bdb_txn_nosync = True
+      elif opt == '--create-option':
+        create_options.append(value)
       elif opt == '--cvs-revnums':
         ctx.svn_property_setters.append(CVSRevisionNumberSetter())
       elif opt == '--mime-types':
@@ -571,7 +605,8 @@ class RunOptions:
         ctx.output_option = ExistingRepositoryOutputOption(target)
       else:
         ctx.output_option = NewRepositoryOutputOption(
-            target, fs_type=fs_type, bdb_txn_nosync=bdb_txn_nosync)
+            target, fs_type=fs_type, bdb_txn_nosync=bdb_txn_nosync,
+            create_options=create_options)
     else:
       ctx.output_option = DumpfileOutputOption(dumpfile)
 
@@ -604,6 +639,9 @@ class RunOptions:
         NormalizePathsSymbolTransform(),
         ])
 
+    if not keep_trivial_imports:
+      symbol_strategy_rules.append(ExcludeTrivialImportBranchRule())
+
     symbol_strategy_rules.append(UnambiguousUsageRule())
     if symbol_strategy_default == 'strict':
       pass
@@ -617,9 +655,6 @@ class RunOptions:
     else:
       assert False
 
-    # Now add a rule that sets the SVN path for each LOD:
-    symbol_strategy_rules.append(DefaultBasePathRule())
-
     # Now add a rule whose job it is to pick the preferred parents of
     # branches and tags:
     symbol_strategy_rules.append(HeuristicPreferredParentRule())
