@@ -16,15 +16,12 @@
 
 """Picklers and unpicklers that are primed with known objects."""
 
-from __future__ import generators
 
 import cStringIO
 import marshal
 import cPickle
 import zlib
 
-from cvs2svn_lib.boolean import *
-
 
 class Serializer:
   """An object able to serialize/deserialize some class of objects."""
