@@ -17,30 +17,17 @@
 """This module contains the SVNCommitCreator class."""
 
 
-from __future__ import generators
-
 import time
 
-from cvs2svn_lib.boolean import *
-from cvs2svn_lib.set_support import *
-from cvs2svn_lib import config
-from cvs2svn_lib.common import warning_prefix
 from cvs2svn_lib.common import InternalError
-from cvs2svn_lib.common import DB_OPEN_NEW
-from cvs2svn_lib.common import DB_OPEN_READ
 from cvs2svn_lib.log import Log
 from cvs2svn_lib.context import Ctx
-from cvs2svn_lib.artifact_manager import artifact_manager
-from cvs2svn_lib.symbol import Branch
-from cvs2svn_lib.database import Database
-from cvs2svn_lib.cvs_item import CVSRevisionDelete
 from cvs2svn_lib.cvs_item import CVSRevisionNoop
 from cvs2svn_lib.cvs_item import CVSBranchNoop
 from cvs2svn_lib.cvs_item import CVSTagNoop
 from cvs2svn_lib.changeset import OrderedChangeset
 from cvs2svn_lib.changeset import BranchChangeset
 from cvs2svn_lib.changeset import TagChangeset
-from cvs2svn_lib.svn_commit import SVNCommit
 from cvs2svn_lib.svn_commit import SVNInitialProjectCommit
 from cvs2svn_lib.svn_commit import SVNPrimaryCommit
 from cvs2svn_lib.svn_commit import SVNPostCommit
