@@ -19,8 +19,6 @@
 
 import bisect
 
-from cvs2svn_lib.boolean import *
-
 from cvs2svn_lib.common import SVN_INVALID_REVNUM
 
 
@@ -56,6 +54,9 @@ class SVNRevisionRange:
     else:
       return '[%d:%d]' % (self.opening_revnum, self.closing_revnum,)
 
+  def __repr__(self):
+    return str(self)
+
 
 class RevisionScores:
   """Represent the scores for a range of revisions."""
