@@ -16,18 +16,12 @@
 
 """SymbolStrategy classes determine how to convert symbols."""
 
-import sys
 import re
 
-from cvs2svn_lib.boolean import *
-from cvs2svn_lib.set_support import *
 from cvs2svn_lib.common import FatalError
-from cvs2svn_lib.common import error_prefix
 from cvs2svn_lib.common import path_join
 from cvs2svn_lib.common import normalize_svn_path
 from cvs2svn_lib.log import Log
-from cvs2svn_lib.context import Ctx
-from cvs2svn_lib.symbol import LineOfDevelopment
 from cvs2svn_lib.symbol import Trunk
 from cvs2svn_lib.symbol import TypedSymbol
 from cvs2svn_lib.symbol import Branch
@@ -98,10 +92,14 @@ class _RegexpStrategyRule(StrategyRule):
 
     self.action = action
 
+  def log(self, symbol):
+    raise NotImplementedError()
+
   def get_symbol(self, symbol, stats):
     if isinstance(symbol, (Trunk, TypedSymbol)):
       return symbol
     elif self.regexp.match(symbol.name):
+      self.log(symbol)
       return self.action(symbol)
     else:
       return symbol
@@ -113,6 +111,12 @@ class ForceBranchRegexpStrategyRule(_RegexpStrategyRule):
   def __init__(self, pattern):
     _RegexpStrategyRule.__init__(self, pattern, Branch)
 
+  def log(self, symbol):
+    Log().verbose(
+        'Converting symbol %s as a branch because it matches regexp "%s".'
+        % (symbol, self.regexp.pattern,)
+        )
+
 
 class ForceTagRegexpStrategyRule(_RegexpStrategyRule):
   """Force symbols matching pattern to be tags."""
@@ -120,6 +124,12 @@ class ForceTagRegexpStrategyRule(_RegexpStrategyRule):
   def __init__(self, pattern):
     _RegexpStrategyRule.__init__(self, pattern, Tag)
 
+  def log(self, symbol):
+    Log().verbose(
+        'Converting symbol %s as a tag because it matches regexp "%s".'
+        % (symbol, self.regexp.pattern,)
+        )
+
 
 class ExcludeRegexpStrategyRule(_RegexpStrategyRule):
   """Exclude symbols matching pattern."""
@@ -127,6 +137,54 @@ class ExcludeRegexpStrategyRule(_RegexpStrategyRule):
   def __init__(self, pattern):
     _RegexpStrategyRule.__init__(self, pattern, ExcludedSymbol)
 
+  def log(self, symbol):
+    Log().verbose(
+        'Excluding symbol %s because it matches regexp "%s".'
+        % (symbol, self.regexp.pattern,)
+        )
+
+
+class ExcludeTrivialImportBranchRule(StrategyRule):
+  """If a symbol is a trivial import branch, exclude it.
+
+  A trivial import branch is defined to be a branch that only had a
+  single import on it (no other kinds of commits) in every file in
+  which it appeared.  In most cases these branches are worthless."""
+
+  def get_symbol(self, symbol, stats):
+    if isinstance(symbol, (Trunk, TypedSymbol)):
+      return symbol
+    if stats.tag_create_count == 0 \
+          and stats.branch_create_count == stats.trivial_import_count:
+      Log().verbose(
+          'Excluding branch %s because it is a trivial import branch.'
+          % (symbol,)
+          )
+      return ExcludedSymbol(symbol)
+    else:
+      return symbol
+
+
+class ExcludeVendorBranchRule(StrategyRule):
+  """If a symbol is a pure vendor branch, exclude it.
+
+  A pure vendor branch is defined to be a branch that only had imports
+  on it (no other kinds of commits) in every file in which it
+  appeared."""
+
+  def get_symbol(self, symbol, stats):
+    if isinstance(symbol, (Trunk, TypedSymbol)):
+      return symbol
+    if stats.tag_create_count == 0 \
+          and stats.branch_create_count == stats.pure_ntdb_count:
+      Log().verbose(
+          'Excluding branch %s because it is a pure vendor branch.'
+          % (symbol,)
+          )
+      return ExcludedSymbol(symbol)
+    else:
+      return symbol
+
 
 class UnambiguousUsageRule(StrategyRule):
   """If a symbol is used unambiguously as a tag/branch, convert it as such."""
@@ -140,8 +198,18 @@ class UnambiguousUsageRule(StrategyRule):
       # Can't decide
       return symbol
     elif is_branch:
+      Log().verbose(
+          'Converting symbol %s as a branch because it is always used '
+          'as a branch.'
+          % (symbol,)
+          )
       return Branch(symbol)
     elif is_tag:
+      Log().verbose(
+          'Converting symbol %s as a tag because it is always used '
+          'as a tag.'
+          % (symbol,)
+          )
       return Tag(symbol)
     else:
       # The symbol didn't appear at all:
@@ -155,6 +223,10 @@ class BranchIfCommitsRule(StrategyRule):
     if isinstance(symbol, (Trunk, TypedSymbol)):
       return symbol
     elif stats.branch_commit_count > 0:
+      Log().verbose(
+          'Converting symbol %s as a branch because there are commits on it.'
+          % (symbol,)
+          )
       return Branch(symbol)
     else:
       return symbol
@@ -170,8 +242,18 @@ class HeuristicStrategyRule(StrategyRule):
     if isinstance(symbol, (Trunk, TypedSymbol)):
       return symbol
     elif stats.tag_create_count >= stats.branch_create_count:
+      Log().verbose(
+          'Converting symbol %s as a tag because it is more often used '
+          'as a tag.'
+          % (symbol,)
+          )
       return Tag(symbol)
     else:
+      Log().verbose(
+          'Converting symbol %s as a branch because it is more often used '
+          'as a branch.'
+          % (symbol,)
+          )
       return Branch(symbol)
 
 
@@ -186,6 +268,10 @@ class AllBranchRule(StrategyRule):
     if isinstance(symbol, (Trunk, TypedSymbol)):
       return symbol
     else:
+      Log().verbose(
+          'Converting symbol %s as a branch because no other rules applied.'
+          % (symbol,)
+          )
       return Branch(symbol)
 
 
@@ -203,63 +289,54 @@ class AllTagRule(StrategyRule):
     if isinstance(symbol, (Trunk, TypedSymbol)):
       return symbol
     else:
+      Log().verbose(
+          'Converting symbol %s as a tag because no other rules applied.'
+          % (symbol,)
+          )
       return Tag(symbol)
 
 
-class DefaultBasePathRule(StrategyRule):
-  """Set LOD base paths to their default values.
+class TrunkPathRule(StrategyRule):
+  """Set the base path for Trunk."""
 
-  The default values are read from the trunk_path, branches_path, and
-  tags_path members of the Project containing the symbol.  If one of
-  these defaults is needed but unset, get_symbol() raises a
-  FatalError."""
+  def __init__(self, trunk_path):
+    self.trunk_path = trunk_path
 
   def get_symbol(self, symbol, stats):
-    if not isinstance(symbol, LineOfDevelopment):
-      # This symbol will not be included in the conversion; skip it.
-      return symbol
-
-    if symbol.base_path is not None:
-      # This lod's base path is already set; leave it.
-      pass
-    elif isinstance(symbol, Trunk):
-      trunk_path = symbol.project.trunk_path
-
-      if trunk_path is None:
-        raise FatalError(
-            'DefaultBasePathRule used for trunk,\n'
-            'but project\'s trunk path is not set'
-            )
-
-      symbol.base_path = trunk_path
-    elif isinstance(symbol, Branch):
-      branches_path = symbol.project.branches_path
-
-      if branches_path is None:
-        raise FatalError(
-            'DefaultBasePathRule used for symbol %s,\n'
-            'but project\'s branch path is not set'
-            % (symbol,)
-            )
-
-      symbol.base_path = path_join(branches_path, symbol.name)
-    elif isinstance(symbol, Tag):
-      tags_path = symbol.project.tags_path
-
-      if tags_path is None:
-        raise FatalError(
-            'DefaultBasePathRule used for symbol %s,\n'
-            'but project\'s tag path is not set'
-            % (symbol,)
-            )
-
-      symbol.base_path = path_join(tags_path, symbol.name)
-    else:
-      raise NotImplementedError()
+    if isinstance(symbol, Trunk) and symbol.base_path is None:
+      symbol.base_path = self.trunk_path
 
     return symbol
 
 
+class SymbolPathRule(StrategyRule):
+  """Set the base paths for symbol LODs."""
+
+  def __init__(self, symbol_type, base_path):
+    self.symbol_type = symbol_type
+    self.base_path = base_path
+
+  def get_symbol(self, symbol, stats):
+    if isinstance(symbol, self.symbol_type) and symbol.base_path is None:
+      symbol.base_path = path_join(self.base_path, symbol.name)
+
+    return symbol
+
+
+class BranchesPathRule(SymbolPathRule):
+  """Set the base paths for Branch LODs."""
+
+  def __init__(self, branch_path):
+    SymbolPathRule.__init__(self, Branch, branch_path)
+
+
+class TagsPathRule(SymbolPathRule):
+  """Set the base paths for Tag LODs."""
+
+  def __init__(self, tag_path):
+    SymbolPathRule.__init__(self, Tag, tag_path)
+
+
 class HeuristicPreferredParentRule(StrategyRule):
   """Use a heuristic rule to pick preferred parents.
 
@@ -292,10 +369,10 @@ class HeuristicPreferredParentRule(StrategyRule):
     if isinstance(symbol, TypedSymbol) and symbol.preferred_parent_id is None:
       preferred_parent = self._get_preferred_parent(stats)
       if preferred_parent is None:
-        Log().debug('%s has no preferred parent' % (symbol,))
+        Log().verbose('%s has no preferred parent' % (symbol,))
       else:
         symbol.preferred_parent_id = preferred_parent.id
-        Log().debug(
+        Log().verbose(
             'The preferred parent of %s is %s' % (symbol, preferred_parent,)
             )
 
@@ -333,6 +410,30 @@ class ManualTrunkRule(StrategyRule):
     return symbol
 
 
+def convert_as_branch(symbol):
+  Log().verbose(
+      'Converting symbol %s as a branch because of manual setting.'
+      % (symbol,)
+      )
+  return Branch(symbol)
+
+
+def convert_as_tag(symbol):
+  Log().verbose(
+      'Converting symbol %s as a tag because of manual setting.'
+      % (symbol,)
+      )
+  return Tag(symbol)
+
+
+def exclude(symbol):
+  Log().verbose(
+      'Excluding symbol %s because of manual setting.'
+      % (symbol,)
+      )
+  return ExcludedSymbol(symbol)
+
+
 class ManualSymbolRule(StrategyRule):
   """Change how particular symbols are converted.
 
@@ -346,9 +447,10 @@ class ManualSymbolRule(StrategyRule):
         affected by this rule.
 
     conversion -- (callable or None) A callable that converts the
-        symbol to its preferred output type.  This should be one of
-        (Branch, Tag, ExcludedSymbol).  If this member is None, then
-        this rule does not affect the symbol's output type.
+        symbol to its preferred output type.  This should normally be
+        one of (convert_as_branch, convert_as_tag, exclude).  If this
+        member is None, then this rule does not affect the symbol's
+        output type.
 
     svn_path -- (str) The SVN path that should be used as the base
         directory for this trunk.  This member must not be None,
@@ -468,9 +570,9 @@ class SymbolHintsFileRule(StrategyRule):
   comment_re = re.compile(r'^(\#|$)')
 
   conversion_map = {
-      'branch' : Branch,
-      'tag' : Tag,
-      'exclude' : ExcludedSymbol,
+      'branch' : convert_as_branch,
+      'tag' : convert_as_tag,
+      'exclude' : exclude,
       '.' : None,
       }
 
