@@ -17,11 +17,6 @@
 """This module contains a class to manage time ranges."""
 
 
-from __future__ import generators
-
-from cvs2svn_lib.boolean import *
-
-
 class TimeRange(object):
   __slots__ = ('t_min', 't_max')
 
