@@ -92,6 +92,12 @@ def setup_pristine_repository():
                            "OUTPUT TREE", expected_output_tree, output_tree)
       sys.exit(1)
 
+    # Finally, disallow any changes to the "pristine" repos.
+    error_msg = "Don't modify the pristine repository"
+    create_failing_hook(main.pristine_dir, 'start-commit', error_msg)
+    create_failing_hook(main.pristine_dir, 'pre-lock', error_msg)
+    create_failing_hook(main.pristine_dir, 'pre-revprop-change', error_msg)
+
 
 ######################################################################
 # Used by every test, so that they can run independently of  one
@@ -635,8 +641,8 @@ def verify_disk(wc_dir_name,
                  singleton_handler_a, a_baton,
                  singleton_handler_b, b_baton,
                  check_props)
-                 
-  
+
+
 
 def run_and_verify_update(wc_dir_name,
                           output_tree, disk_tree, status_tree,
@@ -913,7 +919,7 @@ def run_and_verify_switch(wc_dir_name,
   SINGLETON_HANDLER_B will be passed to tree.compare_trees -- see that
   function's doc string for more details.
 
-  If CHECK_PROPS is set, then disk comparison will examine props. 
+  If CHECK_PROPS is set, then disk comparison will examine props.
 
   Return if successful, raise on failure."""
 
@@ -1147,7 +1153,7 @@ def run_and_verify_diff_summarize_xml(error_re_string = [],
     actual_item = path.getAttribute('item')
     actual_kind = path.getAttribute('kind')
     actual_prop = path.getAttribute('props')
-  
+
     if expected_item != actual_item:
       print "ERROR: expected:", expected_item, "actual:", actual_item
       raise Failure
@@ -1240,7 +1246,7 @@ def run_and_validate_lock(path, username):
 def make_repo_and_wc(sbox, create_wc = True, read_only = False):
   """Create a fresh repository and checkout a wc from it.
 
-  If read_only is False, a dedicated repository will be created, named 
+  If read_only is False, a dedicated repository will be created, named
   TEST_NAME. The repository will live in the global dir 'general_repo_dir'.
   If read_only is True the pristine repository will be used.
 
@@ -1248,7 +1254,7 @@ def make_repo_and_wc(sbox, create_wc = True, read_only = False):
   the repository, named TEST_NAME. The wc directory will live in the global
   dir 'general_wc_dir'.
 
-  Both variables 'general_repo_dir' and 'general_wc_dir' are defined at the 
+  Both variables 'general_repo_dir' and 'general_wc_dir' are defined at the
   top of this test suite.)  Returns on success, raises on failure."""
 
   # Create (or copy afresh) a new repos with a greek tree in it.
@@ -1318,6 +1324,15 @@ def lock_admin_dir(wc_dir):
   path = os.path.join(wc_dir, main.get_admin_name(), 'lock')
   main.file_append(path, "stop looking!")
 
+def create_failing_hook(repo_dir, hook_name, text):
+  """Create a HOOK_NAME hook in REPO_DIR that prints TEXT to stderr and exits
+  with an error."""
+
+  hook_path = os.path.join(repo_dir, 'hooks', hook_name)
+  main.create_python_hook_script(hook_path, 'import sys;\n'
+    'sys.stderr.write("""%%s hook failed: %%s""" %% (%s, %s));\n'
+    'sys.exit(1);\n' % (repr(hook_name), repr(text)))
+
 def enable_revprop_changes(repo_dir):
   """Enable revprop changes in a repository REPOS_DIR by creating a
 pre-revprop-change hook script and (if appropriate) making it executable."""
