@@ -149,10 +149,16 @@ cleanup_mode = False
 # Global variable indicating if svnserve should use Cyrus SASL
 enable_sasl = False
 
+# Global variable indicating that SVNKit binaries should be used
+use_jsvn = False
+
 # Global variable indicating which DAV library, if any, is in use
 # ('neon', 'serf')
 http_library = None
 
+# Configuration file (copied into FSFS fsfs.conf).
+config_file = None
+
 # Global variable indicating what the minor version of the server
 # tested against is (4 for 1.4.x, for example).
 server_minor_version = 5
@@ -264,7 +270,7 @@ def setup_development_mode():
         'run_and_verify_diff_summarize',
         'run_and_verify_diff_summarize_xml',
         'run_and_validate_lock']
-  
+
   for func in l:
     setattr(actions, func, wrap_ex(getattr(actions, func)))
 
@@ -304,6 +310,11 @@ def get_svnserve_conf_file_path(repo_dir):
 
   return os.path.join(repo_dir, "conf", "svnserve.conf")
 
+def get_fsfs_conf_file_path(repo_dir):
+  "Return the path of the fsfs.conf file in REPO_DIR."
+
+  return os.path.join(repo_dir, "db", "fsfs.conf")
+
 # Run any binary, logging the command line and return code
 def run_command(command, error_expected, binary_mode=0, *varargs):
   """Run COMMAND with VARARGS; return exit code as int; stdout, stderr
@@ -430,9 +441,12 @@ def run_command_stdin(command, error_expected, binary_mode=0,
   if verbose_mode:
     stop = time.time()
     print '<TIME = %.6f>' % (stop - start)
+    map(sys.stdout.write, stdout_lines)
+    map(sys.stdout.write, stderr_lines)
 
   if (not error_expected) and (stderr_lines):
-    map(sys.stdout.write, stderr_lines)
+    if not verbose_mode:
+      map(sys.stdout.write, stderr_lines)
     raise Failure
 
   return exit_code, stdout_lines, stderr_lines
@@ -458,14 +472,16 @@ interactive-conflicts = false
 
   # define default server file contents if none provided
   if server_contents is None:
+    http_library_str = ""
     if http_library:
-      server_contents = """
+      http_library_str = "http-library=%s" % (http_library)
+    server_contents = """
 #
 [global]
-http-library=%s
-""" % (http_library)
-    else:
-      server_contents = "#\n"
+%s
+store-plaintext-passwords=yes
+store-passwords=yes
+""" % (http_library_str)
 
   file_write(cfgfile_cfg, config_contents)
   file_write(cfgfile_srv, server_contents)
@@ -623,6 +639,10 @@ def create_repos(path):
     file_append(get_svnserve_conf_file_path(path), "password-db = passwd\n")
     file_append(os.path.join(path, "conf", "passwd"),
                 "[users]\njrandom = rayjandom\njconstant = rayjandom\n");
+
+  if config_file is not None and (fs_type is None or fs_type == 'fsfs'):
+    shutil.copy(config_file, get_fsfs_conf_file_path(path))
+
   # make the repos world-writeable, for mod_dav_svn's sake.
   chmod_tree(path, 0666, 0666)
 
@@ -645,6 +665,7 @@ def copy_repos(src_path, dst_path, head_revision, ignore_uuid = 1):
 
   dump_in, dump_out, dump_err, dump_kid = \
            open_pipe(svnadmin_binary + dump_args, 'b')
+  dump_in.close()
   load_in, load_out, load_err, load_kid = \
            open_pipe(svnadmin_binary + load_args, 'b')
   stop = time.time()
@@ -660,7 +681,6 @@ def copy_repos(src_path, dst_path, head_revision, ignore_uuid = 1):
 
   dump_lines = dump_err.readlines()
   load_lines = load_out.readlines()
-  dump_in.close()
   dump_out.close()
   dump_err.close()
   load_out.close()
@@ -776,27 +796,34 @@ def use_editor(func):
   os.environ['SVNTEST_EDITOR_FUNC'] = func
 
 
-def merge_notify_line(revstart=None, revend=None, same_URL=True):
+def merge_notify_line(revstart=None, revend=None, same_URL=True,
+                      foreign=False):
   """Return an expected output line that describes the beginning of a
   merge operation on revisions REVSTART through REVEND.  Omit both
   REVSTART and REVEND for the case where the left and right sides of
   the merge are from different URLs."""
+  from_foreign_phrase = foreign and "\(from foreign repository\) " or ""
   if not same_URL:
-    return "--- Merging differences between repository URLs into '.+':\n"
+    return "--- Merging differences between %srepository URLs into '.+':\n" \
+           % (foreign and "foreign " or "")
   if revend is None:
     if revstart is None:
       # The left and right sides of the merge are from different URLs.
-      return "--- Merging differences between repository URLs into '.+':\n"
+      return "--- Merging differences between %srepository URLs into '.+':\n" \
+             % (foreign and "foreign " or "")
     elif revstart < 0:
-      return "--- Reverse-merging r%ld into '.+':\n" % abs(revstart)
+      return "--- Reverse-merging %sr%ld into '.+':\n" \
+             % (from_foreign_phrase, abs(revstart))
     else:
-      return "--- Merging r%ld into '.+':\n" % revstart
+      return "--- Merging %sr%ld into '.+':\n" \
+             % (from_foreign_phrase, revstart)
   else:
     if revstart > revend:
-      return "--- Reverse-merging r%ld through r%ld into '.+':\n" % (revstart,
-                                                                     revend)
+      return "--- Reverse-merging %sr%ld through r%ld into '.+':\n" \
+             % (from_foreign_phrase, revstart, revend)
     else:
-      return "--- Merging r%ld through r%ld into '.+':\n" % (revstart, revend)
+      return "--- Merging %sr%ld through r%ld into '.+':\n" \
+             % (from_foreign_phrase, revstart, revend)
 
 
 ######################################################################
@@ -858,6 +885,10 @@ def server_has_partial_replay():
   _check_command_line_parsed()
   return server_minor_version >= 5
 
+def server_enforces_date_syntax():
+  _check_command_line_parsed()
+  return server_minor_version >= 5
+
 
 ######################################################################
 # Sandbox handling
@@ -884,7 +915,7 @@ class Sandbox:
     else:
       self.repo_dir = pristine_dir
       self.repo_url = pristine_url
-      
+
     ### TODO: Move this into to the build() method
     # For dav tests we need a single authz file which must be present,
     # so we recreate it each time a sandbox is created with some default
@@ -1067,6 +1098,14 @@ class TestRunner:
     # Tests that want to use an editor should invoke svntest.main.use_editor.
     os.environ['SVN_EDITOR'] = ''
     os.environ['SVNTEST_EDITOR_FUNC'] = ''
+
+    if use_jsvn:
+      # Set this SVNKit specific variable to the current test (test name plus
+      # its index) being run so that SVNKit daemon could use this test name
+      # for its separate log file
+     os.environ['SVN_CURRENT_TEST'] = os.path.basename(sys.argv[0]) + "_" + \
+                                      str(self.index)
+
     actions.no_sleep_for_timestamps()
 
     saved_dir = os.getcwd()
@@ -1228,6 +1267,7 @@ def usage():
         "                 useful during test development!"
   print " --server-minor-version  Set the minor version for the server.\n" \
         "                 Supports version 4 or 5."
+  print " --config-file   Configuration file for tests."
   print " --help          This information"
 
 
@@ -1254,10 +1294,13 @@ def run_tests(test_list, serial_only = False):
   global svnadmin_binary
   global svnlook_binary
   global svnsync_binary
+  global svndumpfilter_binary
   global svnversion_binary
   global command_line_parsed
   global http_library
+  global config_file
   global server_minor_version
+  global use_jsvn
 
   testnums = []
   # Should the tests be listed (as opposed to executed)?
@@ -1266,13 +1309,14 @@ def run_tests(test_list, serial_only = False):
   parallel = 0
   svn_bin = None
   use_jsvn = False
+  config_file = None
 
   try:
     opts, args = my_getopt(sys.argv[1:], 'vqhpc',
                            ['url=', 'fs-type=', 'verbose', 'quiet', 'cleanup',
                             'list', 'enable-sasl', 'help', 'parallel',
-                            'bin=', 'http-library=', 'server-minor-version=', 
-                            'use-jsvn', 'development'])
+                            'bin=', 'http-library=', 'server-minor-version=',
+                            'use-jsvn', 'development', 'config-file='])
   except getopt.GetoptError, e:
     print "ERROR: %s\n" % e
     usage()
@@ -1342,6 +1386,9 @@ def run_tests(test_list, serial_only = False):
     elif opt == '--development':
       setup_development_mode()
 
+    elif opt == '--config-file':
+      config_file = val
+
   if test_area_url[-1:] == '/': # Normalize url to have no trailing slash
     test_area_url = test_area_url[:-1]
 
@@ -1355,20 +1402,21 @@ def run_tests(test_list, serial_only = False):
     pristine_url = pristine_url.replace('\\', '/')
 
   if use_jsvn:
-    if svn_bin is None: 
+    if svn_bin is None:
       svn_bin = ''
     svn_binary = os.path.join(svn_bin, 'jsvn' + _bat)
     svnadmin_binary = os.path.join(svn_bin, 'jsvnadmin' + _bat)
     svnlook_binary = os.path.join(svn_bin, 'jsvnlook' + _bat)
     svnsync_binary = os.path.join(svn_bin, 'jsvnsync' + _bat)
+    svndumpfilter_binary = os.path.join(svn_bin, 'jsvndumpfilter' + _bat)
     svnversion_binary = os.path.join(svn_bin, 'jsvnversion' + _bat)
-    use_jsvn = False
   else:
     if svn_bin:
       svn_binary = os.path.join(svn_bin, 'svn' + _exe)
       svnadmin_binary = os.path.join(svn_bin, 'svnadmin' + _exe)
       svnlook_binary = os.path.join(svn_bin, 'svnlook' + _exe)
       svnsync_binary = os.path.join(svn_bin, 'svnsync' + _exe)
+      svndumpfilter_binary = os.path.join(svn_bin, 'svndumpfilter' + _exe)
       svnversion_binary = os.path.join(svn_bin, 'svnversion' + _exe)
 
   command_line_parsed = True
