@@ -27,7 +27,12 @@ class TestCase:
   several methods that need to be overridden."""
 
   def __init__(self):
-    self._result_text = ['PASS: ', 'FAIL: ', 'SKIP: ']
+    # Each element is a tuple whose second element indicates benignity:
+    # e.g., True means "can be ignored when in quiet_mode".  See also
+    # XFail.run_test().
+    self._result_text = [('PASS: ', True),
+                         ('FAIL: ', False),
+                         ('SKIP: ', True)]
     self._list_mode_text = ''
 
   def get_description(self):
@@ -151,39 +156,71 @@ class XFail(TestCase):
 
   def run_text(self, result=0):
     if self.cond_func():
-      return ['XFAIL:', 'XPASS:', self.test_case.run_text(2)][result]
+      # Tuple elements mean same as in TestCase._result_text.
+      return [('XFAIL:', True),
+              ('XPASS:', False),
+              self.test_case.run_text(2)][result]
     else:
       return self.test_case.run_text(result)
 
 
 class Skip(TestCase):
-  """A test that will be skipped if condition COND is true."""
+  """A test that will be skipped if its conditional is true."""
+
+  def __init__(self, test_case, cond_func=lambda:1):
+    """Create an Skip instance based on TEST_CASE.  COND_FUNC is a
+    callable that is evaluated at test run time and should return a
+    boolean value.  If COND_FUNC returns true, then TEST_CASE is
+    skipped; otherwise, TEST_CASE is run normally.
+    The evaluation of COND_FUNC is deferred so that it can base its
+    decision on useful bits of information that are not available at
+    __init__ time (like the fact that we're running over a
+    particular RA layer)."""
 
-  def __init__(self, test_case, cond=1):
     TestCase.__init__(self)
     self.test_case = create_test_case(test_case)
-    self.cond = cond
-    if self.cond:
-      self._list_mode_text = 'SKIP'
+    self.cond_func = cond_func
+    try:
+      if self.conditional():
+        self._list_mode_text = 'SKIP'
+    except svntest.Failure:
+      pass
     # Delegate most methods to self.test_case:
     self.get_description = self.test_case.get_description
     self.get_sandbox_name = self.test_case.get_sandbox_name
     self.convert_result = self.test_case.convert_result
+    self.run_text = self.test_case.run_text
 
   def need_sandbox(self):
-    if self.cond:
+    if self.conditional():
       return 0
     else:
       return self.test_case.need_sandbox()
 
   def run(self, sandbox=None):
-    if self.cond:
+    if self.conditional():
       raise svntest.Skip
     elif self.need_sandbox():
       return self.test_case.run(sandbox=sandbox)
     else:
       return self.test_case.run()
 
+  def conditional(self):
+    """Invoke SELF.cond_func(), and return the result evaluated
+    against the expected value."""
+    return self.cond_func()
+
+
+class SkipUnless(Skip):
+  """A test that will be skipped if its conditional is false."""
+
+  def __init__(self, test_case, cond_func):
+    Skip.__init__(self, test_case, cond_func)
+
+  def conditional(self):
+    "Return the negation of SELF.cond_func()."
+    return not self.cond_func()
+
 
 def create_test_case(func):
   if isinstance(func, TestCase):
